@@ -20,7 +20,7 @@
 
 #define NSSEP T('\001')
 
-static void
+static void XMLCALL
 characterData(void *userData, const XML_Char *s, int len)
 {
   FILE *fp = userData;
@@ -113,7 +113,7 @@ attcmp(const void *att1, const void *att2)
   return tcscmp(*(const XML_Char **)att1, *(const XML_Char **)att2);
 }
 
-static void
+static void XMLCALL
 startElement(void *userData, const XML_Char *name, const XML_Char **atts)
 {
   int nAtts;
@@ -137,7 +137,7 @@ startElement(void *userData, const XML_Char *name, const XML_Char **atts)
   puttc(T('>'), fp);
 }
 
-static void
+static void XMLCALL
 endElement(void *userData, const XML_Char *name)
 {
   FILE *fp = userData;
@@ -159,7 +159,7 @@ nsattcmp(const void *p1, const void *p2)
   return tcscmp(att1, att2);
 }
 
-static void
+static void XMLCALL
 startElementNS(void *userData, const XML_Char *name, const XML_Char **atts)
 {
   int nAtts;
@@ -208,7 +208,7 @@ startElementNS(void *userData, const XML_Char *name, const XML_Char **atts)
   puttc(T('>'), fp);
 }
 
-static void
+static void XMLCALL
 endElementNS(void *userData, const XML_Char *name)
 {
   FILE *fp = userData;
@@ -227,7 +227,7 @@ endElementNS(void *userData, const XML_Char *name)
 
 #ifndef W3C14N
 
-static void
+static void XMLCALL
 processingInstruction(void *userData, const XML_Char *target,
                       const XML_Char *data)
 {
@@ -243,57 +243,57 @@ processingInstruction(void *userData, const XML_Char *target,
 
 #endif /* not W3C14N */
 
-static void
-defaultCharacterData(XML_Parser parser, const XML_Char *s, int len)
+static void XMLCALL
+defaultCharacterData(void *userData, const XML_Char *s, int len)
 {
-  XML_DefaultCurrent(parser);
+  XML_DefaultCurrent((XML_Parser) userData);
 }
 
-static void
-defaultStartElement(XML_Parser parser, const XML_Char *name,
+static void XMLCALL
+defaultStartElement(void *userData, const XML_Char *name,
                     const XML_Char **atts)
 {
-  XML_DefaultCurrent(parser);
+  XML_DefaultCurrent((XML_Parser) userData);
 }
 
-static void
-defaultEndElement(XML_Parser parser, const XML_Char *name)
+static void XMLCALL
+defaultEndElement(void *userData, const XML_Char *name)
 {
-  XML_DefaultCurrent(parser);
+  XML_DefaultCurrent((XML_Parser) userData);
 }
 
-static void
-defaultProcessingInstruction(XML_Parser parser, const XML_Char *target,
+static void XMLCALL
+defaultProcessingInstruction(void *userData, const XML_Char *target,
                              const XML_Char *data)
 {
-  XML_DefaultCurrent(parser);
+  XML_DefaultCurrent((XML_Parser) userData);
 }
 
-static void
-nopCharacterData(XML_Parser parser, const XML_Char *s, int len)
+static void XMLCALL
+nopCharacterData(void *userData, const XML_Char *s, int len)
 {
 }
 
-static void
-nopStartElement(XML_Parser parser, const XML_Char *name, const XML_Char **atts)
+static void XMLCALL
+nopStartElement(void *userData, const XML_Char *name, const XML_Char **atts)
 {
 }
 
-static void
-nopEndElement(XML_Parser parser, const XML_Char *name)
+static void XMLCALL
+nopEndElement(void *userData, const XML_Char *name)
 {
 }
 
-static void
-nopProcessingInstruction(XML_Parser parser, const XML_Char *target,
+static void XMLCALL
+nopProcessingInstruction(void *userData, const XML_Char *target,
                          const XML_Char *data)
 {
 }
 
-static void
-markup(XML_Parser parser, const XML_Char *s, int len)
+static void XMLCALL
+markup(void *userData, const XML_Char *s, int len)
 {
-  FILE *fp = XML_GetUserData(parser);
+  FILE *fp = XML_GetUserData((XML_Parser) userData);
   for (; len > 0; --len, ++s)
     puttc(*s, fp);
 }
@@ -313,21 +313,22 @@ metaLocation(XML_Parser parser)
 }
 
 static void
-metaStartDocument(XML_Parser parser)
+metaStartDocument(void *userData)
 {
-  fputts(T("<document>\n"), XML_GetUserData(parser));
+  fputts(T("<document>\n"), XML_GetUserData((XML_Parser) userData));
 }
 
 static void
-metaEndDocument(XML_Parser parser)
+metaEndDocument(void *userData)
 {
-  fputts(T("</document>\n"), XML_GetUserData(parser));
+  fputts(T("</document>\n"), XML_GetUserData((XML_Parser) userData));
 }
 
-static void
-metaStartElement(XML_Parser parser, const XML_Char *name,
+static void XMLCALL
+metaStartElement(void *userData, const XML_Char *name,
                  const XML_Char **atts)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   const XML_Char **specifiedAttsEnd
     = atts + XML_GetSpecifiedAttributeCount(parser);
@@ -358,19 +359,21 @@ metaStartElement(XML_Parser parser, const XML_Char *name,
     fputts(T("/>\n"), fp);
 }
 
-static void
-metaEndElement(XML_Parser parser, const XML_Char *name)
+static void XMLCALL
+metaEndElement(void *userData, const XML_Char *name)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   ftprintf(fp, T("<endtag name=\"%s\""), name);
   metaLocation(parser);
   fputts(T("/>\n"), fp);
 }
 
-static void
-metaProcessingInstruction(XML_Parser parser, const XML_Char *target,
+static void XMLCALL
+metaProcessingInstruction(void *userData, const XML_Char *target,
                           const XML_Char *data)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   ftprintf(fp, T("<pi target=\"%s\" data=\""), target);
   characterData(fp, data, tcslen(data));
@@ -379,9 +382,10 @@ metaProcessingInstruction(XML_Parser parser, const XML_Char *target,
   fputts(T("/>\n"), fp);
 }
 
-static void
-metaComment(XML_Parser parser, const XML_Char *data)
+static void XMLCALL
+metaComment(void *userData, const XML_Char *data)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   fputts(T("<comment data=\""), fp);
   characterData(fp, data, tcslen(data));
@@ -390,27 +394,30 @@ metaComment(XML_Parser parser, const XML_Char *data)
   fputts(T("/>\n"), fp);
 }
 
-static void
-metaStartCdataSection(XML_Parser parser)
+static void XMLCALL
+metaStartCdataSection(void *userData)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   fputts(T("<startcdata"), fp);
   metaLocation(parser);
   fputts(T("/>\n"), fp);
 }
 
-static void
-metaEndCdataSection(XML_Parser parser)
+static void XMLCALL
+metaEndCdataSection(void *userData)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   fputts(T("<endcdata"), fp);
   metaLocation(parser);
   fputts(T("/>\n"), fp);
 }
 
-static void
-metaCharacterData(XML_Parser parser, const XML_Char *s, int len)
+static void XMLCALL
+metaCharacterData(void *userData, const XML_Char *s, int len)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   fputts(T("<chars str=\""), fp);
   characterData(fp, s, len);
@@ -419,35 +426,38 @@ metaCharacterData(XML_Parser parser, const XML_Char *s, int len)
   fputts(T("/>\n"), fp);
 }
 
-static void
-metaStartDoctypeDecl(XML_Parser parser,
+static void XMLCALL
+metaStartDoctypeDecl(void *userData,
                      const XML_Char *doctypeName,
                      const XML_Char *sysid,
                      const XML_Char *pubid,
                      int has_internal_subset)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   ftprintf(fp, T("<startdoctype name=\"%s\""), doctypeName);
   metaLocation(parser);
   fputts(T("/>\n"), fp);
 }
 
-static void
-metaEndDoctypeDecl(XML_Parser parser)
+static void XMLCALL
+metaEndDoctypeDecl(void *userData)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   fputts(T("<enddoctype"), fp);
   metaLocation(parser);
   fputts(T("/>\n"), fp);
 }
 
-static void
-metaNotationDecl(XML_Parser parser,
+static void XMLCALL
+metaNotationDecl(void *userData,
                  const XML_Char *notationName,
                  const XML_Char *base,
                  const XML_Char *systemId,
                  const XML_Char *publicId)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   ftprintf(fp, T("<notation name=\"%s\""), notationName);
   if (publicId)
@@ -462,8 +472,8 @@ metaNotationDecl(XML_Parser parser,
 }
 
 
-static void
-metaEntityDecl(XML_Parser parser,
+static void XMLCALL
+metaEntityDecl(void *userData,
                const XML_Char *entityName,
                int  is_param,
                const XML_Char *value,
@@ -473,6 +483,7 @@ metaEntityDecl(XML_Parser parser,
                const XML_Char *publicId,
                const XML_Char *notationName)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
 
   if (value) {
@@ -505,11 +516,12 @@ metaEntityDecl(XML_Parser parser,
   }
 }
 
-static void
-metaStartNamespaceDecl(XML_Parser parser,
+static void XMLCALL
+metaStartNamespaceDecl(void *userData,
                        const XML_Char *prefix,
                        const XML_Char *uri)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   fputts(T("<startns"), fp);
   if (prefix)
@@ -523,9 +535,10 @@ metaStartNamespaceDecl(XML_Parser parser,
     fputts(T("/>\n"), fp);
 }
 
-static void
-metaEndNamespaceDecl(XML_Parser parser, const XML_Char *prefix)
+static void XMLCALL
+metaEndNamespaceDecl(void *userData, const XML_Char *prefix)
 {
+  XML_Parser parser = (XML_Parser) userData;
   FILE *fp = XML_GetUserData(parser);
   if (!prefix)
     fputts(T("<endns/>\n"), fp);
@@ -533,13 +546,13 @@ metaEndNamespaceDecl(XML_Parser parser, const XML_Char *prefix)
     ftprintf(fp, T("<endns prefix=\"%s\"/>\n"), prefix);
 }
 
-static int
+static int XMLCALL
 unknownEncodingConvert(void *data, const char *p)
 {
   return codepageConvert(*(int *)data, p);
 }
 
-static int
+static int XMLCALL
 unknownEncoding(void *userData, const XML_Char *name, XML_Encoding *info)
 {
   int cp;
@@ -575,7 +588,7 @@ unknownEncoding(void *userData, const XML_Char *name, XML_Encoding *info)
   return 1;
 }
 
-static int
+static int XMLCALL
 notStandalone(void *userData)
 {
   return 0;
@@ -586,12 +599,30 @@ showVersion(XML_Char *prog)
 {
   XML_Char *s = prog;
   XML_Char ch;
+  const XML_Feature *features = XML_GetFeatureList();
   while ((ch = *s) != 0) {
-    if (ch == '/' || ch == '\\')
+    if (ch == '/'
+#ifdef WIN32
+        || ch == '\\'
+#endif
+        )
       prog = s + 1;
     ++s;
   }
   ftprintf(stdout, T("%s using %s\n"), prog, XML_ExpatVersion());
+  if (features != NULL && features[0].feature != XML_FEATURE_END) {
+    int i = 1;
+    ftprintf(stdout, T("%s"), features[0].name);
+    if (features[0].value)
+      ftprintf(stdout, T("=%ld"), features[0].value);
+    while (features[i].feature != XML_FEATURE_END) {
+      ftprintf(stdout, T(", %s"), features[i].name);
+      if (features[i].value)
+        ftprintf(stdout, T("=%ld"), features[i].value);
+      ++i;
+    }
+    ftprintf(stdout, T("\n"));
+  }
 }
 
 static void
